pub struct Engine<'a> {
pub name: Option<Cow<'a, str>>,
pub major: Option<Cow<'a, str>>,
pub minor: Option<Cow<'a, str>>,
pub patch: Option<Cow<'a, str>>,
}Expand description
The layout engine of the product a user agent belongs to.
Fields
name: Option<Cow<'a, str>>The engine name, or None when no pattern matches.
major: Option<Cow<'a, str>>The major version, if the matching pattern captures one.
minor: Option<Cow<'a, str>>The minor version, if the matching pattern captures one.
patch: Option<Cow<'a, str>>The patch version, if the matching pattern captures one.
Implementations
Sourceimpl<'a> Engine<'a>
impl<'a> Engine<'a>
Sourcepub fn version(&self) -> Option<Cow<'_, str>>
pub fn version(&self) -> Option<Cow<'_, str>>
Joins the version parts into a full version string, stopping at the first part which is missing.
use std::borrow::Cow;
use user_agent_parser::Engine;
let engine = Engine {
name: Some(Cow::from("Blink")),
major: Some(Cow::from("57")),
minor: Some(Cow::from("0")),
patch: Some(Cow::from("2987")),
};
assert_eq!(Some(Cow::from("57.0.2987")), engine.version());Sourcepub fn into_owned(self) -> Engine<'static>
pub fn into_owned(self) -> Engine<'static>
Extracts the owned data.