pub struct UserAgentParser { /* private fields */ }Expand description
A set of regular expressions which can extract the product, OS, device, CPU and engine information out of a user-agent string.
Implementations
Sourceimpl UserAgentParser
impl UserAgentParser
Sourcepub fn from_path<P: AsRef<Path>>(
path: P,
) -> Result<UserAgentParser, UserAgentParserError>
pub fn from_path<P: AsRef<Path>>( path: P, ) -> Result<UserAgentParser, UserAgentParserError>
Read the list of regular expressions (YAML data) from a file to create a UserAgentParser instance.
Sourcepub fn from_str<S: AsRef<str>>(
yaml: S,
) -> Result<UserAgentParser, UserAgentParserError>
pub fn from_str<S: AsRef<str>>( yaml: S, ) -> Result<UserAgentParser, UserAgentParserError>
Read the list of regular expressions (YAML data) from a string to create a UserAgentParser instance.
Sourceimpl UserAgentParser
The results borrow from both the parser and the user-agent string, so call into_owned on them to break that tie.
impl UserAgentParser
The results borrow from both the parser and the user-agent string, so call into_owned on them to break that tie.
Sourcepub fn parse_product<'a, S: AsRef<str> + ?Sized>(
&'a self,
user_agent: &'a S,
) -> Product<'a>
pub fn parse_product<'a, S: AsRef<str> + ?Sized>( &'a self, user_agent: &'a S, ) -> Product<'a>
Extracts the product information, falling back to a name of Some("Other") when no pattern matches.
Sourcepub fn parse_os<'a, S: AsRef<str> + ?Sized>(
&'a self,
user_agent: &'a S,
) -> OS<'a>
pub fn parse_os<'a, S: AsRef<str> + ?Sized>( &'a self, user_agent: &'a S, ) -> OS<'a>
Extracts the OS information, falling back to a name of Some("Other") when no pattern matches.
Sourcepub fn parse_device<'a, S: AsRef<str> + ?Sized>(
&'a self,
user_agent: &'a S,
) -> Device<'a>
pub fn parse_device<'a, S: AsRef<str> + ?Sized>( &'a self, user_agent: &'a S, ) -> Device<'a>
Extracts the device information, falling back to a name of Some("Other") when no pattern matches.